import sys
import subprocess

import pip
import requests
import yaml
import click

from oshino_admin import daemon
from oshino_admin.util import parse_plugin

@click.group()
def main():
    pass


@main.command('query')
@click.argument('q')
@click.option('--config', help='Config path', default='config.yaml')
def query(q, config):
    from oshino.config import Config, load
    from riemann_client.client import Client
    cfg = load(config)
    riemann = cfg.riemann
    transport_cls = cfg.riemann.transport
    transport = transport_cls(riemann.host, riemann.port)
    print("Executing query: {0}".format(q))
    with Client(transport) as client:
        print(client.query(q))


@main.group('config')
def config():
    pass


@config.command('init')
@click.argument('path')
def init_config(path):
    default_config = {
        'interval': 10,
        'riemann': {
            'host': 'localhost',
            'port': 5555
        },
        'agents': [
            {
                'name': 'health-check',
                'module': 'oshino.agents.http_agent.HttpAgent',
                'url': 'http://python.org',
                'tag': 'healthcheck'
            }
        ]
    }
    with open(path, 'w') as f:
        yaml.dump(default_config, f, default_flow_style=False)


@main.group('plugin')
def plugin():
    pass


def get_plugins():
    resp = requests.get('https://raw.githubusercontent.com/CodersOfTheNight/'
                        'oshino-admin/master/package_manager/plugins.txt')
    lines = resp.text.split('\n')
    return [parse_plugin(plugin) for plugin in lines
            if len(plugin.rstrip()) > 0]


def validate_plugin(fn):
    def wrapper(*args, **kwargs):
        plugins = get_plugins()
        package = kwargs['package_name']
        if not package in map(lambda x: x[0], plugins):
            print("Invalid plugin '{0}'".format(package))
            sys.exit(1)

        return fn(*args, **kwargs)

    return wrapper


@plugin.command('list')
def plugin_list():
    for package, desc in get_plugins():
        if package in sys.modules.keys():
            print('{0}* - {1}'.format(package, desc))
        else:
            print('{0} - {1}'.format(package, desc))


@plugin.command('install')
@click.argument('package_name')
@validate_plugin
def install(package_name):
    subprocess.call(['pip', 'install', package_name, '--upgrade'])


@plugin.command('uninstall')
@click.argument('package_name')
@validate_plugin
def uninstall(package_name):
    subprocess.call(['pip', 'uninstall', package_name])


@main.command('status')
@click.option('--pid', help='Pid for process', default='/var/run/oshino.pid')
def status(pid):
    daemon.status(pid)


@main.command('start')
@click.option('--config', help='Config path', default='config.yaml')
@click.option('--pid', help='Pid for process', default='/var/run/oshino.pid')
@click.option('--verbose/--normal', help='Verbose', default=False)
@click.option('--logfile', help='Where to put logs', default=None)
def start(config, pid, verbose, logfile):
    print('Starting...')
    daemon.start_daemon(config, pid, verbose, logfile)


@main.command('stop')
@click.option('--pid', help='Pid for process', default='/var/run/oshino.pid')
def stop(pid):
    print('Stopping...')
    daemon.stop_daemon(pid)



if __name__ == "__main__":
    main()



import os
import signal
import logging

from daemonize import Daemonize

logger = logging.getLogger('oshino')

DEFAULT_PID = '/var/run/oshino'


def start_daemon(config_path, pid=DEFAULT_PID, verbose=False, logfile=None):
    from oshino.config import load
    from oshino.core.heart import start_loop

    keep_fds = []
    if logfile:
        level = DEBUG if verbose else INFO
        fh = FileHandler(logfile, level=level)
        logger.handlers.append(fh)
        fh.push_application()
        keep_fds.append(fh.stream.fileno())

    def wrapped():
        cfg = load(config_path)
        return start_loop(cfg)

    daemon = Daemonize(
            app='oshino',
            pid=pid,
            action=wrapped,
            verbose=verbose,
            keep_fds=keep_fds,
            logger=logger,
            chdir=os.getcwd()
    )
    daemon.start()
    return daemon

def stop_daemon(pid=DEFAULT_PID):
    with open(pid, 'r') as f:
        os.kill(int(f.read()), signal.SIGTERM)

def status(pid=DEFAULT_PID):
    if os.path.exists(pid):
        with open(pid, 'r') as f:
            logger.info('Process is running on: {0}'.format(f.read()))
    else:
        logger.error('Process is not running')



import re

PLUGIN_PATT = re.compile(r'(?P<plugin_name>(\w+))\s*=>\s*(?P<plugin_desc>(.*))')

def parse_plugin(plugin_definition):
    try:
        result = PLUGIN_PATT.match(plugin_definition)
        return result.group('plugin_name'), result.group('plugin_desc')
    except Exception as ex:
        print('Failed parsing: "{0}"'.format(plugin_definition))
        raise ex



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['oshino_admin']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'click>=8.0.3,<9.0.0',
 'daemonize>=2.5.0,<3.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'oshino-admin',
    'version': '0.2.0',
    'description': '',
    'long_description': 'oshino-admin\n=============\nAdministration module for Oshino.\nShould come bundled together with Oshino itself.\n\nManaging Plugins\n=================\n`oshino-admin plugin list` - lists all available plugins\n\n`oshino-admin plugin install <plugin_name>` - installs requested plugin\n\n`oshino-admin plugin uninstall <plugin_name>` - uninstalls requested plugin\n\nManaging Oshino\n===============\n\n`oshino-admin start --config=config.yaml --pid=oshino.pid` \n\n`oshino-admin status --pid=oshino.pid` \n\n`oshino-admin stop --pid=oshino.pid` \n\nBe aware, that default PID path `/var/run/oshino.pid` might be unaccessible by oshino-admin (Lacking of root permissions).\nCustom pid path can be defined (as show in example above). Main problem is that you need to direct into correct PID for each service command.\n\nGenerating Config\n==================\n`oshino-admin config init <config_name>.yml`\n\nQuerying metrics\n=================\n`oshino-admin query \'tagged "hw"\' --config=config.yaml`\n\nYou can run any query against Riemann using it\'s query DSL language. Short examples:\n`tagged "<tag>"` - retrieves metrics by tag\n`service = "<something>"` - gives metrics by service name. Same can be done for `host` or other keys\n`service =~ "%<something>%"` - `=~` marks that we\'re going to search by wildcard, `%` marks that there can be anything (similar to SQLs `LIKE %something%`)\n',
    'author': 'Šarūnas Navickas',
    'author_email': 'zaibacu@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/CodersOfTheNight/oshino-admin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
